from tests.utils import _TestBase


class TestNonSCM(_TestBase):
    def test_thLib(self):
        url = self.converter.get_vcs("thLib")
        self.assertInsensitiveEqual(url, "http://work.thaslwanter.at/thlib/html/")

    def test_PyRSS2Gen(self):
        url = self.converter.get_vcs("PyRSS2Gen")
        self.assertInsensitiveEqual(
            url, "http://dalkescientific.com/Python/PyRSS2Gen.html"
        )

    def _test_PyChart(self):
        url = self.converter.get_vcs("PyChart")
        self.assertInsensitiveEqual(
            url, "http://www.hpl.hp.com/personal/yasushi_saito/pychart"
        )
        # Also https://web.archive.org/web/20170225171913/https://gna.org/projects/pychart

    def test_SimpleTAL(self):
        url = self.converter.get_vcs("SimpleTAL")
        self.assertInsensitiveEqual(url, "https://owlfish.com/software/simpleTAL")

    def test_aaronsw_xmltramp(self):
        url = self.converter.get_vcs("xmltramp")
        self.assertInsensitiveEqual(url, "https://www.aaronsw.com/2002/xmltramp")


class TestPyPiMisc(_TestBase):
    def test_basic(self):
        url = self.converter.get_vcs("six")
        self.assertInsensitiveEqual(url, "https://github.com/benjaminp/six")

    def test_repoze_sphinx_autointerface(self):
        url = self.converter.get_vcs("repoze.sphinx.autointerface")
        self.assertInsensitiveEqual(
            url, "https://github.com/repoze/repoze.sphinx.autointerface"
        )

    def test_repoze_sphinx_autointerface_hyphen(self):
        url = self.converter.get_vcs("repoze-sphinx-autointerface")
        self.assertInsensitiveEqual(
            url, "https://github.com/repoze/repoze.sphinx.autointerface"
        )

    def test_py_ext(self):
        url = self.converter.get_vcs("scp")
        self.assertInsensitiveEqual(url, "https://github.com/jbardin/scp.py")

    def test_aexpect(self):
        url = self.converter.get_vcs("aexpect")
        self.assertInsensitiveEqual(url, "https://github.com/avocado-framework/aexpect")

    def test_pythondialog(self):
        url = self.converter.get_vcs("pythondialog")
        self.assertInsensitiveEqual(
            url, "https://sourceforge.net/projects/pythondialog"
        )

    def test_wsgiref(self):
        url = self.converter.get_vcs("wsgiref")
        self.assertInsensitiveEqual(url, "http://svn.eby-sarna.com/wsgiref/")

    def test_s3cmd(self):
        url = self.converter.get_vcs("s3cmd")
        self.assertInsensitiveEqual(url, "https://github.com/s3tools/s3cmd")

    def test_tinys3(self):
        url = self.converter.get_vcs("tinys3")
        self.assertInsensitiveEqual(url, "https://github.com/smore-inc/tinys3")

    def test_github_wiki_pybrain(self):
        url = self.converter.get_vcs("pybrain")
        self.assertInsensitiveEqual(url, "https://github.com/pybrain/pybrain")

    def test_roundup(self):
        url = self.converter.get_vcs("roundup")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/roundup")

    def test_aliyun_direct(self):
        url = self.converter.get_vcs("aliyun-python-sdk-core")
        self.assertInsensitiveEqual(
            url, "https://github.com/aliyun/aliyun-openapi-python-sdk"
        )

    def test_aliyun_namespace(self):
        url = self.converter.get_vcs("aliyun-python-sdk-ecs")
        self.assertInsensitiveEqual(
            url, "https://github.com/aliyun/aliyun-openapi-python-sdk"
        )

    def test_matrix_synapse_ldap3(self):
        url = self.converter.get_vcs("matrix-synapse-ldap3")
        self.assertInsensitiveEqual(
            url, "https://github.com/matrix-org/matrix-synapse-ldap3"
        )

    def test_galaxy_lib(self):
        url = self.converter.get_vcs("galaxy_lib")
        self.assertInsensitiveEqual(url, "https://github.com/galaxyproject/galaxy-lib")

    def test_theme_exclusion_sphinx_rtd_theme(self):
        url = self.converter.get_vcs("sphinx_rtd_theme")
        self.assertInsensitiveEqual(url, "https://github.com/rtfd/sphinx_rtd_theme")
        # redirects to https://github.com/readthedocs/sphinx_rtd_theme

    def test_theme_exclusion_alabaster(self):
        url = self.converter.get_vcs("alabaster")
        self.assertInsensitiveEqual(url, "https://github.com/bitprophet/alabaster")

    def test_trac_xml2rfc(self):
        url = self.converter.get_vcs("xml2rfc")
        self.assertEqual(url, "https://trac.tools.ietf.org/tools/xml2rfc")

    def test_mhash(self):
        url = self.converter.get_vcs("python-mhash")
        # mhash has zero assets
        self.assertEqual(url, "https://sourceforge.net/projects/mhash")

    def test_logbook(self):
        url = self.converter.get_vcs("Logbook")
        self.assertEqual(url, "https://github.com/getlogbook/logbook")

    def test_Pyrex(self):
        url = self.converter.get_vcs("Pyrex")
        self.assertInsensitiveEqual(
            url, "https://www.csse.canterbury.ac.nz/greg.ewing/python/Pyrex/"
        )
        # hg repo at https://www.csse.canterbury.ac.nz/greg.ewing/python/Pyrex/hg/

    def test_tryton_GooCalendar(self):
        url = self.converter.get_vcs("GooCalendar")
        self.assertEqual(url, "https://hg.tryton.org/goocalendar")

    def test_Cython(self):
        url = self.converter.get_vcs("Cython")
        self.assertEqual(url, "https://github.com/cython/cython")

    def test_bpython(self):
        url = self.converter.get_vcs("bpython")
        self.assertEqual(url, "https://github.com/bpython/bpython")

    def test_pygal(self):
        url = self.converter.get_vcs("pygal")
        self.assertEqual(url, "https://github.com/Kozea/pygal")

    def test_amqpstorm(self):
        url = self.converter.get_vcs("amqpstorm")
        self.assertEqual(url, "https://github.com/eandersson/amqpstorm")

    def test_diskcache(self):
        url = self.converter.get_vcs("diskcache")
        self.assertEqual(url, "https://github.com/grantjenks/python-diskcache")

    def test_editdistance(self):
        url = self.converter.get_vcs("editdistance")
        self.assertEqual(url, "https://github.com/aflc/editdistance")

    def test_ftputil(self):
        url = self.converter.get_vcs("ftputil")
        self.assertInsensitiveEqual(url, "https://ftputil.sschwarzer.net/")

    def test_junos_eznc(self):
        url = self.converter.get_vcs("junos-eznc")
        self.assertInsensitiveEqual(url, "https://github.com/Juniper/py-junos-eznc")

    def test_logilab_astng(self):
        url = self.converter.get_vcs("logilab-astng")
        self.assertInsensitiveEqual(
            url, "https://www.logilab.org/project/logilab-astng"
        )

    def test_logilab_common(self):
        url = self.converter.get_vcs("logilab-common")
        self.assertInsensitiveEqual(
            url, "https://www.logilab.org/project/logilab-common"
        )

    def test_logreduce(self):
        # https://github.com/jayvdb/pypidb/issues/46
        url = self.converter.get_vcs("logreduce")
        self.assertInsensitiveEqual(
            url, "https://softwarefactory-project.io/cgit/logreduce"
        )

    def test_Unidecode(self):
        url = self.converter.get_vcs("Unidecode")
        self.assertInsensitiveEqual(
            url, "https://github.com/avian2/unidecode"
        )  # https://www.tablix.org/~avian/git/unidecode.git

    def test_musicbrainzngs(self):
        url = self.converter.get_vcs("musicbrainzngs")
        self.assertInsensitiveEqual(
            url, "https://github.com/alastair/python-musicbrainzngs"
        )

    def test_scm_mercurial(self):
        url = self.converter.get_vcs("mercurial")
        self.assertEqual(url, "https://www.mercurial-scm.org/repo/hg")

    def test_git_scheme(self):
        url = self.converter.get_vcs("lxc-python2")
        self.assertEqual(url, "https://github.com/lxc/python2-lxc")

    def test_git_plus_https_scheme(self):
        url = self.converter.get_vcs("python-zaqarclient")
        self.assertInsensitiveEqual(
            url, "https://opendev.org/openstack/python-zaqarclient"
        )

    def test_git_scheme2(self):
        url = self.converter.get_vcs("tinysegmenter")
        self.assertEqual(url, "http://git.tuxfamily.org/tinysegmente/tinysegmenter/")

    def test_git_scheme_with_backticks(self):
        url = self.converter.get_vcs("package")
        self.assertEqual(url, "https://github.com/ingydotnet/package-py")

    def test_autobahn(self):
        url = self.converter.get_vcs("autobahn")
        self.assertInsensitiveEqual(
            url, "https://github.com/crossbario/autobahn-python"
        )

    def test_travis_svg(self):
        url = self.converter.get_vcs("dropbox")
        self.assertEqual(url, "https://github.com/dropbox/dropbox-sdk-python")

    def test_vlc(self):
        url = self.converter.get_vcs("python-vlc")
        self.assertInsensitiveEqual(
            url, "https://git.videolan.org/?p=vlc/bindings/python.git"
        )

    def test_arch_pymad(self):
        url = self.converter.get_vcs("pymad")
        self.assertEqual(url, "https://spacepants.org/src/pymad")

    def test_www_github_openid_cla(self):
        url = self.converter.get_vcs("python-openid-cla")
        self.assertEqual(url, "https://github.com/puiterwijk/python-openid-cla")

    def test_protego(self):
        url = self.converter.get_vcs("protego")
        self.assertEqual(url, "https://github.com/scrapy/protego")

    def test_dash(self):
        url = self.converter.get_vcs("dash")
        self.assertEqual(url, "https://github.com/plotly/dash")

    def test_strictyaml(self):
        url = self.converter.get_vcs("strictyaml")
        self.assertInsensitiveEqual(url, "https://github.com/crdoconnor/strictyaml")

    def test_tryton_relatorio(self):
        url = self.converter.get_vcs("relatorio")
        self.assertEqual(url, "https://hg.tryton.org/relatorio")

    def test_requests_download(self):
        url = self.converter.get_vcs("requests-download")
        self.assertEqual(url, "https://github.com/takluyver/requests_download")

    def test_rsa(self):
        url = self.converter.get_vcs("rsa")
        self.assertEqual(url, "https://github.com/sybrenstuvel/python-rsa")

    def test_tryton_sql(self):
        url = self.converter.get_vcs("python-sql")
        self.assertEqual(url, "https://hg.tryton.org/python-sql")

    def test_found_whois(self):
        url = self.converter.get_vcs("python-whois")
        self.assertEqual(url, "https://github.com/richardpenman/pywhois")

    def test_selectors2(self):
        url = self.converter.get_vcs("selectors2")
        self.assertEqual(url, "https://github.com/SethMichaelLarson/selectors2")

    def test_sgmllib3k(self):
        url = self.converter.get_vcs("sgmllib3k")
        self.assertEqual(url, "https://hg.hardcoded.net/sgmllib")

    def test_tkreadonly(self):
        url = self.converter.get_vcs("tkreadonly")
        self.assertEqual(url, "https://github.com/pybee/tkreadonly")

    def test_tmx(self):
        url = self.converter.get_vcs("tmx")
        self.assertEqual(url, "https://savannah.nongnu.org/projects/python-tmx")

    def test_yum_urlgrabber(self):
        url = self.converter.get_vcs("urlgrabber")
        self.assertEqual(url, "http://yum.baseurl.org/gitwebd27a.html?p=urlgrabber")

    def test_urwid(self):
        url = self.converter.get_vcs("urwid")
        self.assertEqual(url, "https://github.com/urwid/urwid")

    def test_wsgi_intercept(self):
        url = self.converter.get_vcs("wsgi_intercept")
        self.assertEqual(url, "https://github.com/cdent/wsgi-intercept")

    def test_scales(self):
        url = self.converter.get_vcs("scales")
        self.assertEqual(url, "https://github.com/Cue/scales")

    def test_pymetar(self):
        url = self.converter.get_vcs("pymetar")
        self.assertEqual(url, "https://github.com/klausman/pymetar")

    def test_future(self):
        url = self.converter.get_vcs("future")
        self.assertEqual(url, "https://github.com/PythonCharmers/python-future")

    def test_kaitaistruct(self):
        url = self.converter.get_vcs("kaitaistruct")
        self.assertEqual(url, "https://github.com/kaitai-io/kaitai_struct")
        # actually git submodule https://github.com/kaitai-io/kaitai_struct_python_runtime

    def test_rfc3986(self):
        url = self.converter.get_vcs("rfc3986")
        self.assertInsensitiveEqual(url, "https://github.com/sigmavirus24/rfc3986")
        # redirects to https://github.com/python-hyper/rfc3986

    def test_rfc3987(self):
        url = self.converter.get_vcs("rfc3987")
        self.assertInsensitiveEqual(url, "https://github.com/dgerber/rfc3987")

    def test_Chameleon(self):
        url = self.converter.get_vcs("Chameleon")
        self.assertInsensitiveEqual(url, "https://github.com/malthe/chameleon")

    def test_Genshi(self):
        url = self.converter.get_vcs("Genshi")
        self.assertInsensitiveEqual(url, "https://github.com/edgewall/genshi")

    def test_M2Crypto(self):
        url = self.converter.get_vcs("M2Crypto")
        self.assertInsensitiveEqual(url, "https://gitlab.com/m2crypto/m2crypto")

    def test_twine(self):
        url = self.converter.get_vcs("twine")
        self.assertInsensitiveEqual(url, "https://github.com/pypa/twine")

    def test_WebTest(self):
        url = self.converter.get_vcs("WebTest")
        self.assertInsensitiveEqual(url, "https://github.com/Pylons/webtest")

    def test_pygithub(self):
        url = self.converter.get_vcs("pygithub")
        self.assertInsensitiveEqual(url, "https://github.com/pygithub/pygithub")

    def test_github3py(self):
        url = self.converter.get_vcs("github3.py")
        self.assertInsensitiveEqual(url, "https://github.com/sigmavirus24/github3.py")

    def test_apache_org(self):
        url = self.converter.get_vcs("avro")
        self.assertInsensitiveEqual(url, "https://github.com/apache/avro")

    def test_githubusercontent(self):
        url = self.converter.get_vcs("Kivy")
        self.assertInsensitiveEqual(url, "https://github.com/kivy/kivy")

    def test_github_io_reject(self):
        url = self.converter.get_vcs("IMDbPY")
        self.assertInsensitiveEqual(url, "https://github.com/alberanid/imdbpy")

    def test_github_io_accept(self):
        url = self.converter.get_vcs("acitoolkit")
        self.assertInsensitiveEqual(url, "https://github.com/datacenter/acitoolkit")

    def test_ghbtns_com(self):
        url = self.converter.get_vcs("invoke")
        self.assertInsensitiveEqual(url, "https://github.com/pyinvoke/invoke")

    def test_codecov_io(self):
        url = self.converter.get_vcs("multidict")
        self.assertInsensitiveEqual(url, "https://github.com/aio-libs/multidict")

    def test_unittest2(self):
        url = self.converter.get_vcs("unittest2")
        self.assertInsensitiveEqual(url, "https://hg.python.org/unittest2")

    def test_heapdict(self):
        url = self.converter.get_vcs("heapdict")
        self.assertInsensitiveEqual(url, "https://github.com/DanielStutzbach/heapdict")

    def test_keyczar(self):
        url = self.converter.get_vcs("python-keyczar")
        self.assertInsensitiveEqual(url, "https://github.com/google/keyczar")

    def test_dateutil(self):
        url = self.converter.get_vcs("python-dateutil")
        self.assertInsensitiveEqual(url, "https://github.com/dateutil/dateutil")

    def test_invocations(self):
        url = self.converter.get_vcs("invocations")
        self.assertInsensitiveEqual(url, "https://github.com/pyinvoke/invocations")

    def test_gitorious_org_oset(self):
        url = self.converter.get_vcs("oset")
        self.assertInsensitiveEqual(url, "http://gitorious.org/sleipnir/python-oset")

    def test_celery_kombu(self):
        url = self.converter.get_vcs("kombu")
        self.assertInsensitiveEqual(url, "https://github.com/celery/kombu")

    def test_lftools(self):
        url = self.converter.get_vcs("lftools")
        self.assertInsensitiveEqual(
            url, "https://gerrit.linuxfoundation.org/infra/q/project:releng%252Flftools"
        )

    def test_custom_host_bsddb3(self):
        url = self.converter.get_vcs("bsddb3")
        self.assertInsensitiveEqual(url, "https://hg.jcea.es/pybsddb")

    def test_sf_net_crcmod(self):
        url = self.converter.get_vcs("crcmod")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/crcmod")

    def test_sf_net_expanded_docutils(self):
        url = self.converter.get_vcs("docutils")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/docutils")

    def test_ZSI(self):
        url = self.converter.get_vcs("ZSI")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/pywebsvcs")

    def test_ignore_sf_djvulibre(self):
        url = self.converter.get_vcs("python-djvulibre")
        self.assertInsensitiveEqual(url, "https://github.com/jwilk/python-djvulibre")

    def test_pycdio(self):
        url = self.converter.get_vcs("pycdio")
        self.assertInsensitiveEqual(
            url, "http://git.savannah.gnu.org/cgit/libcdio/pycdio.git"
        )

    def test_webpy(self):
        url = self.converter.get_vcs("web.py")
        self.assertInsensitiveEqual(url, "https://github.com/webpy/webpy")

    def test_spark_parser(self):
        url = self.converter.get_vcs("spark-parser")
        self.assertInsensitiveEqual(url, "https://github.com/rocky/python-spark")

    def test_tagpy(self):
        url = self.converter.get_vcs("tagpy")
        self.assertInsensitiveEqual(url, "http://git.tiker.net/?p=tagpy.git")

    def test_git_plus_https_asyncssh(self):
        url = self.converter.get_vcs("asyncssh")
        self.assertInsensitiveEqual(url, "https://github.com/ronf/asyncssh")

    def test_catkin_pkg(self):
        url = self.converter.get_vcs("catkin-pkg")
        self.assertInsensitiveEqual(
            url, "https://github.com/ros-infrastructure/catkin_pkg"
        )

    def test_imagecodecs(self):
        url = self.converter.get_vcs("imagecodecs")
        self.assertInsensitiveEqual(url, "https://github.com/cgohlke/imagecodecs")

    def test_podcastparser(self):
        url = self.converter.get_vcs("podcastparser")
        self.assertInsensitiveEqual(url, "https://github.com/gpodder/podcastparser")

    def test_pluggy(self):
        url = self.converter.get_vcs("pluggy")
        self.assertInsensitiveEqual(url, "https://github.com/pytest-dev/pluggy")

    def test_debian(self):
        url = self.converter.get_vcs("python-debian")
        self.assertInsensitiveEqual(
            url, "https://salsa.debian.org/python-debian-team/python-debian"
        )

    def test_nxapi_plumbing(self):
        url = self.converter.get_vcs("nxapi-plumbing")
        self.assertInsensitiveEqual(url, "https://github.com/ktbyers/nxapi-plumbing")

    def test_proboscis(self):
        url = self.converter.get_vcs("proboscis")
        self.assertInsensitiveEqual(
            url, "https://github.com/rackerlabs/python-proboscis"
        )

    def test_pysndfile(self):
        url = self.converter.get_vcs("pysndfile")
        self.assertInsensitiveEqual(url, "https://forge-2.ircam.fr/roebel/pysndfile")

    def test_coveralls_check(self):
        url = self.converter.get_vcs("coveralls-check")
        self.assertInsensitiveEqual(url, "https://github.com/cjw296/coveralls-check")

    def test_MarkupSafe(self):
        url = self.converter.get_vcs("MarkupSafe")
        self.assertInsensitiveEqual(url, "https://github.com/pallets/MarkupSafe")

    def test_pygraphviz(self):
        url = self.converter.get_vcs("pygraphviz")
        self.assertInsensitiveEqual(url, "https://github.com/pygraphviz/pygraphviz")

    def test_jsonpickle(self):
        url = self.converter.get_vcs("jsonpickle")
        self.assertInsensitiveEqual(url, "https://github.com/jsonpickle/jsonpickle")

    def test_django_oot(self):
        url = self.converter.get_vcs("django-oot")
        self.assertInsensitiveEqual(url, "http://tracpub.yaco.es/djangoapps/wiki/OOT")

    def test_django_oauth(self):
        url = self.converter.get_vcs("django-oauth")
        self.assertInsensitiveEqual(url, "http://code.welldev.org/django-oauth/")

    def test_oauth_provider(self):
        url = self.converter.get_vcs("oauth_provider")
        self.assertInsensitiveEqual(url, "http://code.welldev.org/django-oauth/")

    def test_django_oauth_plus(self):
        url = self.converter.get_vcs("django-oauth-plus")
        self.assertInsensitiveEqual(
            url, "https://bitbucket.org/david/django-oauth-plus"
        )

    def test_moneta(self):
        url = self.converter.get_vcs("moneta")
        self.assertInsensitiveEqual(url, "https://github.com/d9pouces/Moneta")

    def test_github_octolytics(self):
        url = self.converter.get_vcs("github_octolytics")
        self.assertInsensitiveEqual(
            url, "https://github.com/andrewp-as-is/github-octolytics.py"
        )

    def test_qpid_proton(self):
        url = self.converter.get_vcs("python-qpid-proton")
        self.assertInsensitiveEqual(url, "https://github.com/apache/qpid-proton")

    def test_DeCiDa(self):
        url = self.converter.get_vcs("DeCiDa")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/decida")

    def test_sf_net_trac_AllPairs(self):
        url = self.converter.get_vcs("AllPairs")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/allpairs")

    def test_requests_threads(self):
        url = self.converter.get_vcs("requests-threads")
        self.assertInsensitiveEqual(url, "https://github.com/requests/requests-threads")

    def test_openmesh(self):
        url = self.converter.get_vcs("openmesh")
        self.assertInsensitiveEqual(
            url, "https://www.graphics.rwth-aachen.de:9000/OpenMesh/openmesh-python"
        )

    def test_threevis(self):
        url = self.converter.get_vcs("threevis")
        self.assertInsensitiveEqual(
            url, "https://www.graphics.rwth-aachen.de:9000/threevis/threevis"
        )

    def test_linaro(self):
        url = self.converter.get_vcs("django-restricted-resource")
        self.assertInsensitiveEqual(
            url, "https://git.linaro.org/lava/django-restricted-resource.git"
        )

    def test_wmi(self):
        url = self.converter.get_vcs("wmi")
        self.assertInsensitiveEqual(url, "https://github.com/tjguk/wmi")

    def test_pywin32(self):
        url = self.converter.get_vcs("pywin32")
        self.assertInsensitiveEqual(url, "https://github.com/mhammond/pywin32")

    def test_waiting(self):
        url = self.converter.get_vcs("waiting")
        self.assertInsensitiveEqual(url, "https://github.com/vmalloc/waiting")

    def test_flux(self):
        url = self.converter.get_vcs("flux")
        self.assertInsensitiveEqual(url, "https://github.com/vmalloc/flux")

    def test_flex(self):
        url = self.converter.get_vcs("flex")
        self.assertInsensitiveEqual(url, "https://github.com/pipermerriam/flex")

    def test_awscli(self):
        url = self.converter.get_vcs("awscli")
        self.assertInsensitiveEqual(url, "https://github.com/aws/aws-cli")

    def test_pep8(self):
        url = self.converter.get_vcs("pep8")
        self.assertInsensitiveEqual(url, "https://github.com/pycqa/pep8")

    def test_igraph(self):
        url = self.converter.get_vcs(
            "python-igraph"
        )  # not http://www.lfd.uci.edu/~gohlke/pythonlibs
        self.assertInsensitiveEqual(url, "https://github.com/igraph/python-igraph")

    def test_gsutil(self):
        url = self.converter.get_vcs("gsutil")
        self.assertInsensitiveEqual(
            url, "https://github.com/GoogleCloudPlatform/gsutil"
        )

    def test_requests_toolbelt(self):
        url = self.converter.get_vcs("requests-toolbelt")
        self.assertInsensitiveEqual(url, "https://github.com/requests/toolbelt")

    def test_snowplow_tracker(self):
        url = self.converter.get_vcs("snowplow-tracker")
        self.assertInsensitiveEqual(
            url, "https://github.com/snowplow/snowplow-python-tracker"
        )

    def test_snowplow_tracker_minimal(self):
        url = self.converter.get_vcs("minimal-snowplow-tracker")
        self.assertInsensitiveEqual(
            url, "https://github.com/fishtown-analytics/snowplow-python-tracker"
        )

    def test_Schevo(self):
        url = self.converter.get_vcs("Schevo")
        self.assertInsensitiveEqual(url, "https://github.com/11craft/schevo")

    def test_SchevoDurus(self):
        url = self.converter.get_vcs("SchevoDurus")
        self.assertInsensitiveEqual(url, "https://github.com/11craft/schevodurus")

    def test_SchevoGears(self):
        url = self.converter.get_vcs("SchevoGears")
        self.assertInsensitiveEqual(url, "https://github.com/11craft/schevogears")

    def test_SchevoSql(self):
        url = self.converter.get_vcs("SchevoSql")
        self.assertInsensitiveEqual(url, "https://github.com/11craft/schevosql")

    def test_publicsuffix2(self):
        url = self.converter.get_vcs("publicsuffix2")
        self.assertInsensitiveEqual(url, "https://github.com/nexb/python-publicsuffix2")

    def test_publicsuffixlist(self):
        url = self.converter.get_vcs("publicsuffixlist")
        self.assertInsensitiveEqual(url, "https://github.com/ko-zu/psl")

    def test_pattern(self):
        url = self.converter.get_vcs("pattern")
        self.assertInsensitiveEqual(url, "https://github.com/clips/pattern")

    def test_pip(self):
        url = self.converter.get_vcs("pip")
        self.assertInsensitiveEqual(url, "https://github.com/pypa/pip")

    def test_pipfile(self):
        url = self.converter.get_vcs("pipfile")
        self.assertInsensitiveEqual(url, "https://github.com/pypa/pipfile")

    def test_geopandas(self):
        url = self.converter.get_vcs("geopandas")
        self.assertEqual(url, "https://github.com/geopandas/geopandas")

    def test_upt(self):
        url = self.converter.get_vcs("upt")
        self.assertEqual(url, "https://framagit.org/upt/upt")

    def test_upt_pypi(self):
        url = self.converter.get_vcs("upt-pypi")
        self.assertEqual(url, "https://framagit.org/upt/upt-pypi")

    def test_flup(self):
        url = self.converter.get_vcs("flup")
        self.assertEqual(url, "https://www.saddi.com/software/flup/")


class TestEmail(_TestBase):
    def test_sympy(self):
        url = self.converter.get_vcs("sympy")
        self.assertInsensitiveEqual(url, "https://github.com/sympy/sympy")

    def test_pypowervm(self):
        url = self.converter.get_vcs("pypowervm")
        self.assertInsensitiveEqual(url, "https://github.com/powervm/pypowervm")

    def test_sigopt(self):
        url = self.converter.get_vcs("sigopt")
        self.assertInsensitiveEqual(url, "https://github.com/sigopt/sigopt-python")

    def test_veusz(self):
        url = self.converter.get_vcs("veusz")
        self.assertInsensitiveEqual(url, "https://github.com/veusz/veusz")

    def test_pystemmer(self):
        url = self.converter.get_vcs("pystemmer")
        self.assertInsensitiveEqual(url, "https://github.com/snowballstem/pystemmer")

    def test_medusa(self):
        url = self.converter.get_vcs("medusa")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/oedipus")

    def test_igor(self):
        url = self.converter.get_vcs("igor")
        self.assertInsensitiveEqual(url, "http://git.tremily.us/?p=igor.git")


class TestOfflineWeb(_TestBase):
    def test_potr(self):
        url = self.converter.get_vcs("python-potr")
        self.assertInsensitiveEqual(url, "https://github.com/afflux/pure-python-otr")


class TestWeighting(_TestBase):
    def test_pyload(self):
        url = self.converter.get_vcs("pyload-ng")
        self.assertEqual(url, "https://github.com/pyload/pyload")

    def test_featureflow(self):
        url = self.converter.get_vcs("featureflow")
        self.assertEqual(url, "https://github.com/JohnVinyard/featureflow")


class TestSimilarity(_TestBase):
    def test_pycli(self):
        url = self.converter.get_vcs("pycli")
        self.assertEqual(url, "https://github.com/whilp/cli")

    def test_mygpoclient(self):
        url = self.converter.get_vcs("mygpoclient")
        self.assertInsensitiveEqual(url, "https://github.com/gpodder/mygpoclient")

    def test_agate_excel(self):
        url = self.converter.get_vcs("agate-excel")
        self.assertInsensitiveEqual(url, "https://github.com/wireservice/agate-excel")

    def test_sst(self):
        url = self.converter.get_vcs("sst")
        self.assertInsensitiveEqual(url, "https://launchpad.net/selenium-simple-test")

    def test_boxsdk(self):
        url = self.converter.get_vcs("boxsdk")
        self.assertInsensitiveEqual(url, "https://github.com/box/box-python-sdk")


class TestMulti(_TestBase):
    def test_bytecodeassembler(self):
        url = self.converter.get_vcs("BytecodeAssembler")
        self.assertInsensitiveEqual(
            url, "https://github.com/peak-legacy/bytecodeassembler"
        )
        # Was "http://svn.eby-sarna.com/BytecodeAssembler/"

    def test_peak(self):
        url = self.converter.get_vcs("PEAK")
        self.assertInsensitiveEqual(url, "https://github.com/peak-legacy/peak")
        # Was "http://svn.eby-sarna.com/PEAK/"

    def test_oslo_vmware(self):
        url = self.converter.get_vcs("oslo-vmware")
        self.assertInsensitiveEqual(url, "https://opendev.org/openstack/oslo.vmware")

    def test_oslo_upgradecheck(self):
        url = self.converter.get_vcs("oslo-upgradecheck")
        self.assertInsensitiveEqual(
            url, "https://opendev.org/openstack/oslo.upgradecheck"
        )

    def test_drfdocs(self):
        url = self.converter.get_vcs("drfdocs")
        self.assertInsensitiveEqual(
            url, "https://github.com/manosim/django-rest-framework-docs"
        )

    def test_uritemplate(self):
        url = self.converter.get_vcs("uritemplate-py")
        self.assertInsensitiveEqual(url, "https://github.com/python-hyper/uritemplate")

    def test_category_encoders(self):
        url = self.converter.get_vcs("category-encoders")
        self.assertInsensitiveEqual(
            url, "https://github.com/scikit-learn-contrib/categorical-encoding"
        )

    def test_google_api_core(self):
        url = self.converter.get_vcs("google-api-core")
        self.assertInsensitiveEqual(
            url, "https://github.com/googleapis/google-cloud-python"
        )

    def test_googleapis_common_protos(self):
        url = self.converter.get_vcs("googleapis-common-protos")
        self.assertInsensitiveEqual(url, "https://github.com/googleapis/googleapis")

    def test_lecli(self):
        url = self.converter.get_vcs("logentries-lecli")
        self.assertInsensitiveEqual(url, "https://github.com/logentries/lecli")

    def test_pyload_ng(self):
        url = self.converter.get_vcs("pyload-ng")
        self.assertInsensitiveEqual(url, "https://github.com/pyload/pyload")
        # also mirrored to "https://gitlab.com/pyload/pyload"



import io

import unidiff
from logging_helper import setup_logging

from ._url_extract import _url_extractor_wrapper

logger = setup_logging()


def _get_raw_patch_url(url):
    assert url.startswith("https://github.com/") or url.endswith(".patch"), url
    if "/pull/" in url:
        slug = url[len("https://github.com/") : url.find("/pull/")]
        pull_id = url.rsplit("/", 1)[1]
        url = "https://patch-diff.githubusercontent.com/raw/{}/pull/{}.patch".format(
            slug, pull_id
        )
    else:
        assert url.endswith(".patch"), url

    return url


def _get_patch_redirects(patch, allow_add_only=False):
    f = io.StringIO(patch.decode("utf-8"))
    p = unidiff.PatchSet(f)
    redirect_mappings = []

    for p_file in p:
        for p_hunk in p_file:
            removed_urls = []
            added_urls = []
            for p_line in p_hunk:
                if p_line.line_type == "-":
                    removed_urls += _url_extractor_wrapper(str(p_line))
                    logger.debug("removed_urls: {}".format(removed_urls))
                elif p_line.line_type == "+":
                    added_urls += _url_extractor_wrapper(str(p_line))
                    logger.debug("added_urls: {}".format(added_urls))

            for url in added_urls.copy():
                if url in removed_urls:
                    removed_urls.remove(url)
                    added_urls.remove(url)
                elif not url.startswith("http://") and not url.startswith("https://"):
                    added_urls.remove(url)

            if not allow_add_only:
                if removed_urls:
                    assert added_urls
                elif added_urls:
                    assert removed_urls

            if not removed_urls and not allow_add_only:
                logger.info("Skipping url patch {} {}".format(removed_urls, added_urls))
                continue

            if len(set(added_urls)) == 1:
                to_url = added_urls[0]
                if allow_add_only and not removed_urls:
                    removed_urls.append(None)

                for url in removed_urls:
                    redirect_mappings.append((url, to_url))
            elif not added_urls:
                for url in removed_urls:
                    redirect_mappings.append((url, None))
            else:
                if len(added_urls) != len(removed_urls):
                    logger.info(
                        "Hunk ignored as removed cant be mapped to added: {}\n{}".format(
                            removed_urls, added_urls
                        )
                    )
                    continue
                for i, to_url in enumerate(added_urls):
                    redirect_mappings.append((removed_urls[i], to_url))

    return redirect_mappings



from tests.utils import _TestBase


class TestGitHubRepo(_TestBase):
    def test_sphinxcontrib_infrae(self):
        rv = self._check_github_repo("sphinx-doc/sphinxcontrib-infrae")
        self.assertFalse(rv)

    def test_sphinxcontrib_napoleon(self):
        rv = self._check_github_repo("sphinx-contrib/napoleon")
        self.assertTrue(rv)

    def test_sphinxcontrib_websupport(self):
        rv = self._check_github_repo("sphinx-doc/sphinxcontrib-websupport")
        self.assertTrue(rv)

    def test_dedupe_variable_datetime(self):
        url = self._get_scm("dedupe-variable-datetime")
        self.assertInsensitiveEqual(
            url, "https://github.com/datamade/dedupe-variable-datetime"
        )
        rv = self._check_github_repo(url)
        self.assertFalse(rv)

    def test_nosegae(self):
        url = self._get_scm("nosegae")
        self.assertInsensitiveEqual(url, "https://github.com/Trii/NoseGAE")

    def test_fpdf(self):
        url = self._get_scm("fpdf")
        self.assertInsensitiveEqual(url, "https://github.com/reingart/pyfpdf")

    def test_flask_compress(self):
        url = self._get_scm("Flask-Compress")
        self.assertInsensitiveEqual(
            url, "https://github.com/colour-science/flask-compress"
        )
        rv = self._check_github_repo(url)
        self.assertTrue(rv)

    def test_temporary_fork_repo_metafone(self):
        url = self._get_scm("metafone")
        self.assertInsensitiveEqual(url, "https://github.com/al45tair/metaphone")
        rv = self._check_github_repo(url)
        self.assertFalse(rv)

    def test_pycountry_convert(self):
        url = self._get_scm("pycountry-convert")
        self.assertInsensitiveEqual(
            url, "https://github.com/jefftune/pycountry-convert"
        )
        rv = self._check_github_repo(url)
        self.assertTrue(rv)

    def test_bb_Adamanteus(self):
        url = self._get_scm("Adamanteus")
        self.assertInsensitiveEqual(url, "https://bitbucket.org/Josh/adamanteus")
        rv = self._check_github_repo(url)
        self.assertFalse(rv)

    def test_bb_anyvc(self):
        url = self._get_scm("anyvc")
        self.assertInsensitiveEqual(
            url, "https://github.com/ronnypfannschmidt-attic/anyvc"
        )
        rv = self._check_github_repo(url)
        self.assertTrue(rv)

    def test_sshmount_netrc(self):
        url = self._get_scm("sshmount-netrc")
        self.assertInsensitiveEqual(
            url, "https://github.com/tjaartvdwalt/sshmount-netrc"
        )
        rv = self._check_github_repo(url)
        self.assertTrue(rv)

    def test_coveralls_check(self):
        rv = self._check_github_repo("https://github.com/cjw296/coverage-check")
        self.assertFalse(rv)

    def test_hypothesis_pytest(self):
        url = self._get_scm("hypothesis-pytest")
        self.assertInsensitiveEqual(url, "https://github.com/DRMacIver/hypothesis")
        repo = self._check_github_repo(url)
        self.assertTrue(repo)
        # TODO: resolve redirect to https://github.com/HypothesisWorks/hypothesis

    def test_vcs_mirrors(self):
        url = self._get_scm("vcs-mirrors")
        self.assertInsensitiveEqual(url, "https://github.com/pcdummy/vcs-mirrors")
        rv = self._check_github_repo(url)
        self.assertTrue(rv)

    def test_flake8_todo(self):
        url = self._get_scm("flake8-todo")
        self.assertInsensitiveEqual(url, "https://github.com/schlamar/flake8-todo")
        rv = self._check_github_repo(url)
        self.assertTrue(rv)

    def test_flake8_immediate(self):
        url = self._get_scm("flake8-immediate")
        self.assertInsensitiveEqual(url, "https://github.com/schlamar/flake8-immediate")
        rv = self._check_github_repo(url)
        self.assertTrue(rv)

        url = self._get_scm("flake8_immediate")
        self.assertInsensitiveEqual(url, "https://github.com/schlamar/flake8-immediate")
        rv = self._check_github_repo(url)
        self.assertTrue(rv)


class TestGitHubMissingRepo(_TestBase):
    def test_debian(self):
        url = self._get_scm("debian")
        self.assertInsensitiveEqual(url, "https://github.com/ourway/iran")
        rv = self._check_github_repo(url)
        self.assertFalse(rv)

    def test_nonexistant_repo_azureml(self):
        url = self._get_scm("azureml-model-management-sdk")
        self.assertInsensitiveEqual(
            url, "https://github.com/Azure/azure-ml-api-sdk-python"
        )
        rv = self._check_github_repo(url)
        self.assertFalse(rv)

    def test_nonexistant_repo_octohot(self):
        url = self._get_scm("octohot")
        self.assertInsensitiveEqual(url, "https://github.com/hotmart-org/octohot")
        rv = self._check_github_repo(url)
        self.assertFalse(rv)

    def test_nonexistant_repo_spotinst_agent(self):
        url = self._get_scm("spotinst-agent")
        self.assertInsensitiveEqual(
            url, "https://github.com/spotinst/spotinst-spectrum-agent"
        )
        rv = self._check_github_repo(url)
        self.assertFalse(rv)


class TestGitHubSetupPy(_TestBase):
    def test_putty(self):
        rv = self._check_github_setuppy("jayvdb/flake8-putty", "flake8-putty")
        self.assertTrue(rv)
        self.assertIn("putty", rv)



from unittest_expander import expand, foreach

from tests.data import exact, mismatch, missing_repos
from tests.utils import _TestBase, normalize, web_session

expected = {}
expected.update(exact)
expected.update(mismatch)


class TestSpecific(_TestBase):
    def test_pydpi(self):
        url = self.converter.get_vcs("pydpi")
        self.assertEqual(url, "https://code.google.com/p/pydpi")

    def test_gmpy(self):
        url = self.converter.get_vcs("gmpy")
        self.assertEqual(url, "https://code.google.com/p/gmpy")

    def test_dicompyler(self):
        url = self.converter.get_vcs("dicompyler")
        self.assertEqual(url, "https://code.google.com/p/dicompyler")

    def test_socksipy_branch(self):
        url = self.converter.get_vcs("socksipy-branch")
        self.assertEqual(url, "https://code.google.com/p/socksipy-branch")

    def test_pyp(self):
        url = self.converter.get_vcs("pyp")
        self.assertEqual(url, "https://code.google.com/p/pyp")

    def test_pyopt(self):
        url = self.converter.get_vcs("pyopt")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/pyopt")

    def test_euclid(self):
        url = self.converter.get_vcs("euclid")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/pyeuclid")

    def test_crc16(self):
        url = self.converter.get_vcs("crc16")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/pycrc16")
        # TODO: should be https://github.com/gennady/pycrc16

    def test_nosetty(self):
        url = self.converter.get_vcs("nosetty")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/nosetty")

    def test_pysvg(self):
        url = self.converter.get_vcs("pysvg")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/pysvg")

    def test_lepl(self):
        url = self.converter.get_vcs("lepl")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/lepl")

    def test_nmap(self):
        url = self.converter.get_vcs("python-nmap")
        # Read meta redirect to go to https://bitbucket.org/xael/python-nmap
        self.assertInsensitiveEqual(url, "https://code.google.com/p/python-nmap")

    def test_fixture(self):
        url = self.converter.get_vcs("fixture")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/fixture")

    def test_leveldb(self):
        url = self.converter.get_vcs("leveldb")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/py-leveldb")

    def test_ntplib(self):
        url = self.converter.get_vcs("ntplib")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/ntplib")

    def test_pysmell(self):
        url = self.converter.get_vcs("pysmell")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/pysmell")

    def test_publicsuffix(self):
        url = self.converter.get_vcs("publicsuffix")
        self.assertInsensitiveEqual(
            url, "https://code.google.com/p/python-public-suffix-list"
        )

    def test_prettytable(self):
        url = self.converter.get_vcs("prettytable")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/prettytable")

    def test_mglob(self):
        url = self.converter.get_vcs("mglob")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/vvtools")

    def test_apache_extras_cpl(self):
        url = self.converter.get_vcs("cql")
        self.assertInsensitiveEqual(
            url, "https://code.google.com/a/apache-extras.org/p/cassandra-dbapi2"
        )

    def test_gdist(self):
        url = self.converter.get_vcs("gdist")
        self.assertNotEqual(url, "https://github.com/the-virtual-brain/tvb-geodesic")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/geodesic")

    def test_django_pagination(self):
        url = self.converter.get_vcs("django-pagination")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/django-pagination")

    def test_django_gravatar(self):
        url = self.converter.get_vcs("django-gravatar")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/django-gravatar")

    def test_django_swaps(self):
        url = self.converter.get_vcs("django-swaps")
        self.assertInsensitiveEqual(url, "https://code.google.com/p/django-swaps")


class TestRedirectOther(_TestBase):
    def test_mox3(self):
        url = self.converter.get_vcs("mox3")
        self.assertInsensitiveEqual(url, "https://opendev.org/openstack/mox3")

    def test_slixmpp(self):
        url = self.converter.get_vcs("slixmpp")
        self.assertInsensitiveEqual(url, "https://lab.louiz.org/poezio/slixmpp")


class TestRedirectGitHub(_TestBase):
    def test_dpkt(self):
        url = self.converter.get_vcs("dpkt")
        self.assertInsensitiveEqual(url, "https://github.com/kbandla/dpkt")

    def test_beatbox(self):
        url = self.converter.get_vcs("beatbox")
        self.assertInsensitiveEqual(url, "https://github.com/superfell/Beatbox")

    def test_pydot2(self):
        url = self.converter.get_vcs("pydot2")
        self.assertInsensitiveEqual(url, "https://github.com/erocarrera/pydot")

    def test_pymilter(self):
        url = self.converter.get_vcs("pymilter")
        self.assertEqual(url, "https://github.com/sdgathman/pymilter")

    def test_cppclean(self):
        url = self.converter.get_vcs("cppclean")
        self.assertInsensitiveEqual(url, "https://github.com/myint/cppclean")

    def test_ply(self):
        url = self.converter.get_vcs("ply")
        self.assertInsensitiveEqual(url, "https://github.com/dabeaz/ply")

    def test_pydot(self):
        url = self.converter.get_vcs("pydot")
        self.assertInsensitiveEqual(url, "https://github.com/pydot/pydot")

    def test_protorpc(self):
        url = self.converter.get_vcs("protorpc")
        self.assertInsensitiveEqual(url, "https://github.com/google/protorpc")

    def test_ibmdb(self):
        url = self.converter.get_vcs("ibm-db")
        self.assertInsensitiveEqual(url, "https://github.com/ibmdb/python-ibmdb")

    def test_ibmdbsa(self):
        url = self.converter.get_vcs("ibm-db-sa")
        self.assertInsensitiveEqual(url, "https://github.com/ibmdb/python-ibmdb")

    def test_scikits_sparse(self):
        url = self.converter.get_vcs("scikits-sparse")
        self.assertInsensitiveEqual(
            url, "https://github.com/scikit-sparse/scikit-sparse"
        )

    def test_pysphere(self):
        url = self.converter.get_vcs("pysphere")
        self.assertInsensitiveEqual(url, "https://github.com/argos83/pysphere")

    def test_cronwatch(self):
        url = self.converter.get_vcs("cronwatch")
        self.assertInsensitiveEqual(url, "https://github.com/wdlowry/cronwatch")

    def test_murmurhash3(self):
        url = self.converter.get_vcs("murmurhash3")
        self.assertInsensitiveEqual(url, "https://github.com/veegee/mmh3")

    def test_iniparse(self):
        url = self.converter.get_vcs("iniparse")
        self.assertInsensitiveEqual(url, "https://github.com/candlepin/python-iniparse")

    def test_progressbar(self):
        url = self.converter.get_vcs("progressbar")
        self.assertInsensitiveEqual(
            url, "https://github.com/niltonvolpato/python-progressbar"
        )

    def test_yara(self):
        url = self.converter.get_vcs("yara")
        self.assertInsensitiveEqual(url, "https://github.com/mjdorma/yara-ctypes")

    def test_webapp(self):
        url = self.converter.get_vcs("webapp")
        self.assertInsensitiveEqual(url, "https://github.com/alectramell/webapp")

    def test_webapp2(self):
        url = self.converter.get_vcs("webapp2")
        self.assertInsensitiveEqual(
            url, "https://github.com/googlecloudplatform/webapp2"
        )

    def test_tvb_gdist(self):
        url = self.converter.get_vcs("tvb-gdist")
        self.assertInsensitiveEqual(
            url, "https://github.com/the-virtual-brain/tvb-geodesic"
        )

    def test_mpmath(self):
        url = self.converter.get_vcs("mpmath")
        self.assertInsensitiveEqual(url, "https://github.com/fredrik-johansson/mpmath")

    def test_scitools(self):
        url = self.converter.get_vcs("SciTools")
        self.assertInsensitiveEqual(url, "https://github.com/hplgit/scitools")

    def test_django_jython(self):
        url = self.converter.get_vcs("django-jython")
        self.assertInsensitiveEqual(
            url, "https://github.com/beachmachine/django-jython"
        )


@expand
class TestExplicit(_TestBase):

    expected = expected
    names = [name for name, url in expected.items() if "code.google.com" in url]
    expected_failures = []

    @foreach(names)
    def test_package(self, name):
        expected = self.expected[name]
        try:
            url = self.converter.get_vcs(name)
        except Exception:
            if name in self.expected_failures:
                return
            raise

        if name in self.expected_failures:
            return

        self.assertIsNotNone(url)
        self.assertInsensitiveEqual(url, expected)

        normalised_name = normalize(name)

        if normalised_name in missing_repos:
            return
        r = web_session.get(url)
        r.raise_for_status()



import urlextract

from ._html import get_html_hrefs

try:
    _url_extractor = urlextract.URLExtract(cache_dns=False)
except Exception:
    _url_extractor = urlextract.URLExtract()

MAX_URLS = 1000


def _url_extractor_base(content):
    try:
        return _url_extractor.gen_urls(content, check_dns=True)
    except Exception:
        return _url_extractor.gen_urls(content)


def _url_extractor_wrapper(content, url=None):
    extractor = _url_extractor_base(content)
    seen = set()
    for url in extractor:
        # package and cffi end up with repo``
        # https://github.com/lipoja/URLExtract/issues/13
        url = url.strip("`")

        if url.endswith(".html."):  # lit
            url = url[:-1]

        yield url

        seen.add(url)
        if len(seen) == MAX_URLS:
            return


def _url_extractor_wrapper_no_dns(content, url):
    return _url_extractor.find_urls(content, only_unique=True, check_dns=False)


def _url_no_extract(content, url):
    return []


def _url_extract_both(content, url):
    for url in get_html_hrefs(content, url):
        yield url
    for url in _url_extractor_wrapper(content, url):
        yield url



from pypidb._pypi import InvalidPackage, PackageWithoutUrls
from tests.utils import _TestBase


class TestAntlr(_TestBase):
    def test_antlr_3(self):
        url = self.converter.get_vcs("antlr-python-runtime")
        self.assertInsensitiveEqual(url, "https://github.com/antlr/antlr3")

    def test_antlr3_hyphen(self):
        url = self.converter.get_vcs("antlr3-python-runtime")
        self.assertInsensitiveEqual(url, "https://github.com/antlr/antlr3")

    def test_antlr3_underscore(self):
        url = self.converter.get_vcs("antlr3_python_runtime")
        self.assertInsensitiveEqual(url, "https://github.com/antlr/antlr3")

    def test_antlr4(self):
        url = self.converter.get_vcs("antlr4-python3-runtime")
        self.assertInsensitiveEqual(url, "https://github.com/antlr/antlr4")

    def test_antlr4python3runtime(self):
        self.assertRaisesNoUrls("antlr4python3runtime")


class TestGRPC(_TestBase):
    def test_grpc(self):
        url = self.converter.get_vcs("grpc")
        self.assertInsensitiveEqual(url, "https://bitbucket.org/seewind/grpc")

    def test_grpcio(self):
        url = self.converter.get_vcs("grpcio")
        self.assertInsensitiveEqual(url, "https://github.com/grpc/grpc")

    def test_grpcio_tools(self):
        url = self.converter.get_vcs("grpcio-tools")
        self.assertInsensitiveEqual(url, "https://github.com/grpc/grpc")

    def test_grpcio_testing(self):
        url = self.converter.get_vcs("grpcio-testing")
        self.assertInsensitiveEqual(url, "https://github.com/grpc/grpc")

    def test_grpcio_reflection(self):
        url = self.converter.get_vcs("grpcio-reflection")
        self.assertInsensitiveEqual(url, "https://github.com/grpc/grpc")

    def test_grpcio_status(self):
        url = self.converter.get_vcs("grpcio-status")
        self.assertInsensitiveEqual(url, "https://github.com/grpc/grpc")

    def test_grpcio_health_checking(self):
        url = self.converter.get_vcs("grpcio-health-checking")
        self.assertInsensitiveEqual(url, "https://github.com/grpc/grpc")

    def test_grpcio_tests(self):
        with self.assertRaises(InvalidPackage):
            self.converter.get_vcs("grpcio-tests")

    def test_grpcio_gcp(self):
        url = self.converter.get_vcs("grpcio-gcp")
        self.assertInsensitiveEqual(
            url, "https://github.com/GoogleCloudPlatform/grpc-gcp-python"
        )

    def test_grpcio_opentracing(self):
        # https://github.com/opentracing-contrib/python-grpc/pull/27
        url = self.converter.get_vcs("grpcio-opentracing")
        self.assertInsensitiveEqual(
            url, "https://github.com/opentracing-contrib/python-grpc"
        )


class TestBuildbot(_TestBase):
    def test_buildbot(self):
        url = self.converter.get_vcs("buildbot")
        self.assertInsensitiveEqual(url, "https://github.com/buildbot/buildbot")

    def test_buildbot_worker(self):
        url = self.converter.get_vcs("buildbot-worker")
        self.assertInsensitiveEqual(url, "https://github.com/buildbot/buildbot")

    def test_buildbot_slave(self):
        # slave was renamed to worker
        url = self.converter.get_vcs("buildbot-slave")
        self.assertInsensitiveEqual(url, "https://github.com/buildbot/buildbot")


class TestZope(_TestBase):
    def test_zope(self):
        url = self.converter.get_vcs("zope")
        self.assertInsensitiveEqual(url, "https://github.com/zopefoundation/Zope")

    def test_zc(self):
        with self.assertRaises(InvalidPackage):
            self.converter.get_vcs("zc")

    def test_z3c(self):
        with self.assertRaises(InvalidPackage):
            self.converter.get_vcs("z3c")

    def test_zope_testing(self):
        url = self.converter.get_vcs("zope.testing")
        self.assertEqual(url, "https://github.com/zopefoundation/zope.testing")

    def test_zdaemon(self):
        url = self.converter.get_vcs("zdaemon")
        self.assertEqual(url, "https://github.com/zopefoundation/zdaemon")

    def test_zc_zdaemonrecipe(self):
        url = self.converter.get_vcs("zc.zdaemonrecipe")
        self.assertEqual(url, "https://github.com/zopefoundation/zc.zdaemonrecipe")

    def test_zc_recipe_deployment(self):
        url = self.converter.get_vcs("zc.recipe.deployment")
        self.assertEqual(url, "https://github.com/zopefoundation/zc.recipe.deployment")

    def test_zc_customdoctests(self):
        url = self.converter.get_vcs("zc.customdoctests")
        self.assertEqual(url, "https://github.com/zopefoundation/zc.customdoctests")

    def test_zc_customdoctests_hyphen(self):
        url = self.converter.get_vcs("zc-customdoctests")
        self.assertEqual(url, "https://github.com/zopefoundation/zc.customdoctests")

    def test_zc_customdoctests_underscore(self):
        url = self.converter.get_vcs("zc_customdoctests")
        self.assertEqual(url, "https://github.com/zopefoundation/zc.customdoctests")

    def test_zc_recipe_egg(self):
        url = self.converter.get_vcs("zc.recipe.egg")
        self.assertEqual(url, "https://github.com/buildout/buildout")

    def test_zc_buildout(self):
        url = self.converter.get_vcs("zc.buildout")
        self.assertEqual(url, "https://github.com/buildout/buildout")

    def test_zc_zodbrecipes(self):
        url = self.converter.get_vcs("zc.zodbrecipes")
        self.assertEqual(url, "https://github.com/zopefoundation/zc.zodbrecipes")

    def test_z3c_schema2xml(self):
        url = self.converter.get_vcs("z3c.schema2xml")
        self.assertEqual(url, "https://github.com/zopefoundation/z3c.schema2xml")

    def test_z3c_testsetup(self):
        url = self.converter.get_vcs("z3c.testsetup")
        self.assertEqual(url, "https://github.com/zopefoundation/z3c.testsetup")


class TestTurboGears(_TestBase):
    def test_turbogears_v1(self):
        url = self.converter.get_vcs("turbogears")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/turbogears1")

    def test_TurboKid(self):
        url = self.converter.get_vcs("turbokid")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/turbogears1")

    def test_turbomail(self):
        url = self.converter.get_vcs("turbomail")
        self.assertInsensitiveEqual(url, "https://github.com/marrow/marrow.mailer")
        # marrow/marrow.mailer redirects to marrow/mailer
        url = self.converter.get_vcs("marrow.mailer")
        self.assertInsensitiveEqual(url, "https://github.com/marrow/mailer")

    def test_Cheetah(self):
        url = self.converter.get_vcs("Cheetah")
        self.assertInsensitiveEqual(url, "https://github.com/cheetahtemplate/cheetah")

    def test_Cheetah3(self):
        url = self.converter.get_vcs("Cheetah3")
        self.assertInsensitiveEqual(url, "https://github.com/CheetahTemplate3/cheetah3")

    def test_TurboCheetah(self):
        url = self.converter.get_vcs("TurboCheetah")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/turbogears1")

    def test_turbogears_v2(self):
        url = self.converter.get_vcs("turbogears2")
        self.assertInsensitiveEqual(url, "https://github.com/TurboGears/tg2")

    def test_tg_devtools(self):
        url = self.converter.get_vcs("tg.devtools")
        self.assertInsensitiveEqual(url, "https://github.com/TurboGears/tg2devtools")

    def test_sprox(self):
        url = self.converter.get_vcs("sprox")
        self.assertInsensitiveEqual(url, "https://bitbucket.org/percious/sprox")
        # should be https://github.com/TurboGears/sprox

    def test_tgext_debugbar(self):
        url = self.converter.get_vcs("tgext.debugbar")
        self.assertInsensitiveEqual(url, "https://github.com/turbogears/tgext.debugbar")

    def test_tg_ext_silverplate(self):
        url = self.converter.get_vcs("tg.ext.silverplate")
        self.assertInsensitiveEqual(url, "https://github.com/pedersen/tgtools")
        # Should stop at https://code.google.com/p/tgtools, as github repo is empty

    def test_Catwalk(self):
        url = self.converter.get_vcs("Catwalk")
        self.assertInsensitiveEqual(url, "https://github.com/pedersen/tgtools")

    def test_tgmochikit(self):
        url = self.converter.get_vcs("tgmochikit")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/turbogears1")

    def test_tgfastdata(self):
        self.assertRaisesNoUrls("tgfastdata")
        # Probably should be https://sourceforge.net/projects/turbogears1
        # but too old to be of great concern

    def test_ToscaWidgets(self):
        # TODO: https://bitbucket.org/toscawidgets/toscawidgets
        self.assertRaisesNoUrls("ToscaWidgets")

    def test_tw2_core(self):
        url = self.converter.get_vcs("tw2.core")
        self.assertInsensitiveEqual(url, "https://github.com/toscawidgets/tw2.core")

    def test_tw_forms(self):
        self.assertRaisesNoUrls("tw.forms")
        # Need link queue chooser to get "http://bitbucket.org/toscawidgets/tw.forms"

    def test_tw_dynforms(self):
        self.assertRaisesNoUrls("tw.dynforms")
        # Need link queue chooser to get "http://bitbucket.org/toscawidgets/tw.dynforms"


class TestMozilla(_TestBase):
    # mozbase moved from github to hg.mozilla.org/mozilla-central , which is good
    # for cloning but not helpful for 'project SCM'.

    def test_mozlog(self):
        url = self.converter.get_vcs("mozlog")
        self.assertInsensitiveEqual(
            url, "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase"
        )

    def test_mozrunner(self):
        url = self.converter.get_vcs("mozrunner")
        self.assertInsensitiveEqual(
            url, "https://wiki.mozilla.org/Auto-tools/Projects/Mozbase"
        )

    def test_nss(self):
        url = self.converter.get_vcs("python-nss")
        self.assertInsensitiveEqual(url, "https://hg.mozilla.org/projects/python-nss")

    def _test_marionette_driver(self):
        url = self.converter.get_vcs("marionette-driver")
        self.assertInsensitiveEqual(
            url, "https://wiki.mozilla.org/Auto-tools/Projects/Marionette"
        )
        # vcs is https://hg.mozilla.org/mozilla-central/file/tip/testing/marionette/client


class TestSTScI(_TestBase):
    def test_relic(self):
        url = self.converter.get_vcs("relic")
        self.assertInsensitiveEqual(url, "https://github.com/spacetelescope/relic")

    def test_stscipython(self):
        self.assertRaisesNoUrls("stscipython")

    def test_pyraf(self):
        url = self.converter.get_vcs("pyraf")
        self.assertInsensitiveEqual(url, "https://github.com/spacetelescope/pyraf")

    def test_drizzlepac(self):
        url = self.converter.get_vcs("drizzlepac")
        self.assertInsensitiveEqual(url, "https://github.com/spacetelescope/drizzlepac")

    def _test_hstcal(self):
        # Not on pypi
        url = self.converter.get_vcs("hstcal")
        self.assertInsensitiveEqual(url, "https://github.com/spacetelescope/hstcal")

    def test_stsci_sphinxext(self):
        url = self.converter.get_vcs("stsci.sphinxext")
        self.assertInsensitiveEqual(
            url, "https://github.com/spacetelescope/stsci.sphinxext"
        )

    def test_stsci_distutils(self):
        url = self.converter.get_vcs("stsci.distutils")
        self.assertInsensitiveEqual(
            url, "https://github.com/spacetelescope/stsci.distutils"
        )

    def test_PyFITS(self):
        url = self.converter.get_vcs("PyFITS")
        self.assertInsensitiveEqual(url, "https://github.com/spacetelescope/PyFITS")


class TestXStatic(_TestBase):
    def test_openstack_jquery_tablesorter(self):
        url = self._get_scm("XStatic-JQuery-TableSorter")
        self.assertInsensitiveEqual(
            url, "https://github.com/openstack/XStatic-JQuery.TableSorter"
        )

    def test_takluyver_term_dot_js(self):
        url = self._get_scm("XStatic-term.js")
        self.assertInsensitiveEqual(url, "https://github.com/takluyver/XStatic-termjs")

    def test_ThomasWaldmann_jquery_ui(self):
        url = self._get_scm("XStatic-jquery-ui")
        self.assertInsensitiveEqual(
            url, "https://github.com/xstatic-py/xstatic-jquery_ui"
        )

    def test_r1chardj0n3s_smart_table(self):
        url = self._get_scm("XStatic-smart-table")
        self.assertInsensitiveEqual(
            url, "https://github.com/r1chardj0n3s/xstatic-smart-table"
        )

    def test_dmsimard_patternfly(self):
        url = self._get_scm("XStatic-patternfly")
        self.assertInsensitiveEqual(url, "https://github.com/python-xstatic/patternfly")

    def test_dmsimard_objectpath(self):
        url = self._get_scm("XStatic-objectpath")
        self.assertInsensitiveEqual(
            url, "https://github.com/dmsimard/python-XStatic-objectpath-distgit"
        )

    def test_dmsimard_tv4(self):
        url = self._get_scm("XStatic-tv4")
        self.assertInsensitiveEqual(
            url, "https://github.com/dmsimard/python-XStatic-tv4-distgit"
        )

    def test_dmsimard_Angular_Schema_Form(self):
        url = self._get_scm("XStatic-Angular-Schema-Form")
        self.assertInsensitiveEqual(
            url,
            "https://github.com/dmsimard/python-XStatic-angular-schema-form-distgit",
        )

    def test_Spin(self):
        url = self._get_scm("XStatic-Spin")
        self.assertInsensitiveEqual(url, "https://github.com/openstack/XStatic-spin")

    def test_jquery_quicksearch(self):
        url = self._get_scm("XStatic-jquery-quicksearch")
        self.assertInsensitiveEqual(
            url, "https://github.com/openstack/xstatic-jquery.quicksearch"
        )


class TestSphinx(_TestBase):
    def test_sphinx(self):
        url = self.converter.get_vcs("sphinx")
        self.assertEqual(url, "https://github.com/sphinx-doc/sphinx")

    def test_sphinxcontrib_programoutput(self):
        url = self.converter.get_vcs("sphinxcontrib-programoutput")
        self.assertInsensitiveEqual(
            url, "https://github.com/NextThought/sphinxcontrib-programoutput"
        )

    def test_sphinxcontrib_openapi(self):
        url = self.converter.get_vcs("sphinxcontrib-openapi")
        self.assertEqual(url, "https://github.com/ikalnytskyi/sphinxcontrib-openapi")

    def test_sphinxcontrib_versioning(self):
        url = self.converter.get_vcs("sphinxcontrib-versioning")
        self.assertEqual(url, "https://github.com/Robpol86/sphinxcontrib-versioning")

    def test_sphinxcontrib_plantuml(self):
        url = self.converter.get_vcs("sphinxcontrib-plantuml")
        self.assertEqual(url, "https://github.com/sphinx-contrib/plantuml")

    def test_sphinxcontrib_napoleon(self):
        url = self.converter.get_vcs("sphinxcontrib-napoleon")
        self.assertInsensitiveEqual(url, "https://github.com/sphinx-contrib/napoleon")

    def test_sphinxcontrib_domaintools(self):
        url = self.converter.get_vcs("sphinxcontrib-domaintools")
        self.assertEqual(url, "https://bitbucket.org/klorenz/sphinxcontrib-domaintools")

    def test_sphinxcontrib_images(self):
        url = self.converter.get_vcs("sphinxcontrib-images")
        self.assertEqual(url, "https://github.com/sphinx-contrib/images")

    def test_sphinxcontrib_youtube(self):
        url = self.converter.get_vcs("sphinxcontrib-youtube")
        self.assertEqual(url, "https://github.com/shomah4a/sphinxcontrib.youtube")

    def test_sphinxcontrib_proof(self):
        url = self.converter.get_vcs("sphinxcontrib-proof")
        self.assertEqual(url, "https://framagit.org/spalax/sphinxcontrib-proof")

    def test_sphinxcontrib_spelling(self):
        url = self.converter.get_vcs("sphinxcontrib-spelling")
        self.assertEqual(url, "https://github.com/sphinx-contrib/spelling")

    def test_sphinxcontrib_restbuilder(self):
        url = self.converter.get_vcs("sphinxcontrib-restbuilder")
        self.assertEqual(url, "https://github.com/sphinx-contrib/restbuilder")

    def test_sphinxcontrib_runcmd(self):
        url = self.converter.get_vcs("sphinxcontrib-runcmd")
        self.assertEqual(url, "https://github.com/invenia/sphinxcontrib-runcmd")

    def test_sphinxcontrib_collations(self):
        with self.assertRaises(InvalidPackage):
            self.converter.get_vcs("sphinxcontrib-collations")

    def test_sphinxcontrib_websupport(self):
        url = self.converter.get_vcs("sphinxcontrib-websupport")
        self.assertEqual(url, "https://github.com/sphinx-doc/sphinxcontrib-websupport")

    def test_sphinxcontrib_applehelp(self):
        url = self.converter.get_vcs("sphinxcontrib-applehelp")
        self.assertInsensitiveEqual(
            url, "https://github.com/sphinx-doc/sphinxcontrib-applehelp"
        )

    def test_sphinxcontrib_infrae(self):
        self.assertRaisesNoUrls("sphinxcontrib-infrae")

    def test_sphinxcontrib_whoosh(self):
        # https://github.com/sphinx-contrib/whooshindex
        with self.assertRaises(PackageWithoutUrls):
            self.converter.get_vcs("sphinxcontrib-whoosh")

    def test_sphinxcontrib_gruffygen(self):
        # https://github.com/hhatto/sphinxcontrib-gruffygen
        with self.assertRaises(PackageWithoutUrls):
            self.converter.get_vcs("sphinxcontrib-gruffygen")

    def test_sphinxcontrib_devhelp(self):
        url = self.converter.get_vcs("sphinxcontrib-devhelp")
        self.assertEqual(url, "https://github.com/sphinx-doc/sphinxcontrib-devhelp")

    def test_sphinxcontrib_htmlhelp(self):
        url = self.converter.get_vcs("sphinxcontrib-htmlhelp")
        self.assertEqual(url, "https://github.com/sphinx-doc/sphinxcontrib-htmlhelp")

    def test_sphinxcontrib_jsmath(self):
        url = self.converter.get_vcs("sphinxcontrib-jsmath")
        self.assertEqual(url, "https://github.com/sphinx-doc/sphinxcontrib-jsmath")

    def test_sphinxcontrib_qthelp(self):
        url = self.converter.get_vcs("sphinxcontrib-qthelp")
        self.assertEqual(url, "https://github.com/sphinx-doc/sphinxcontrib-qthelp")

    def test_sphinxcontrib_serializinghtml(self):
        url = self.converter.get_vcs("sphinxcontrib-serializinghtml")
        self.assertEqual(
            url, "https://github.com/sphinx-doc/sphinxcontrib-serializinghtml"
        )


class TestPyPiRedirectSpin(_TestBase):
    def test_jupyter_console(self):
        # sends pypi into a spin
        # https://github.com/ionrock/cachecontrol/issues/214
        url = self.converter.get_vcs("jupyter-console")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_console")

    def test_dot_jupyter_console(self):
        url = self.converter.get_vcs("jupyter.console")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_console")

    def test_django_coverage_plugin(self):
        url = self.converter.get_vcs("django-coverage-plugin")
        self.assertInsensitiveEqual(
            url, "https://github.com/nedbat/django_coverage_plugin"
        )


class TestJupyter(_TestBase):
    def test_jupyter(self):
        url = self.converter.get_vcs("jupyter")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter")

    def test_jupyter_core(self):
        url = self.converter.get_vcs("jupyter-core")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_core")

    def test_jupyter_client(self):
        url = self.converter.get_vcs("jupyter-client")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_client")

    def _test_jupyter_console(self):
        url = self.converter.get_vcs("jupyter-console")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_console")

    def _test_jupyter_console_underscore(self):
        url = self.converter.get_vcs("jupyter_console")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_console")

    def test_jupyter_protocol(self):
        url = self.converter.get_vcs("jupyter-protocol")
        self.assertInsensitiveEqual(
            url, "https://github.com/takluyver/jupyter_protocol"
        )

    def test_jupyter_server(self):
        url = self.converter.get_vcs("jupyter-server")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_server")

    def test_nbformat(self):
        url = self.converter.get_vcs("nbformat")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/nbformat")

    def test_jupyter_nbformat(self):
        url = self.converter.get_vcs("jupyter_nbformat")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/jupyter_nbformat")

    def test_nbdime(self):
        url = self.converter.get_vcs("nbdime")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter/nbdime")

    def test_ipython(self):
        url = self.converter.get_vcs("ipython")
        self.assertInsensitiveEqual(url, "https://github.com/ipython/ipython")

    def test_ipynb(self):
        url = self.converter.get_vcs("ipynb")
        self.assertInsensitiveEqual(url, "https://github.com/ipython/ipynb")

    def test_traitlets(self):
        url = self.converter.get_vcs("traitlets")
        self.assertInsensitiveEqual(url, "https://github.com/ipython/traitlets")

    def test_rlipython(self):
        url = self.converter.get_vcs("rlipython")
        self.assertInsensitiveEqual(url, "https://github.com/ipython/rlipython")

    def test_ipykernel(self):
        url = self.converter.get_vcs("ipykernel")
        self.assertInsensitiveEqual(url, "https://github.com/ipython/ipykernel")

    def test_ipyparallel(self):
        url = self.converter.get_vcs("ipyparallel")
        self.assertInsensitiveEqual(url, "https://github.com/ipython/ipyparallel")

    def test_ipywidgets(self):
        url = self.converter.get_vcs("ipywidgets")
        self.assertInsensitiveEqual(url, "https://github.com/ipython/ipywidgets")

    def test_ipyleaflet(self):
        url = self.converter.get_vcs("ipyleaflet")
        self.assertInsensitiveEqual(
            url, "https://github.com/jupyter-widgets/ipyleaflet"
        )

    def test_ipysheet(self):
        # https://github.com/jupyter-widgets/ipysheet
        self.assertRaisesNoUrls("ipysheet")

    def test_ipymidicontrols(self):
        url = self.converter.get_vcs("ipymidicontrols")
        self.assertInsensitiveEqual(
            url, "https://github.com/jupyter-widgets/midicontrols"
        )

    def test_traittypes(self):
        url = self.converter.get_vcs("traittypes")
        self.assertInsensitiveEqual(
            url, "https://github.com/jupyter-widgets/traittypes"
        )

    def test_sidecar(self):
        url = self.converter.get_vcs("sidecar")
        self.assertInsensitiveEqual(
            url, "https://github.com/jupyter-widgets/jupyterlab-sidecar"
        )

    def test_pythreejs(self):
        url = self.converter.get_vcs("pythreejs")
        self.assertInsensitiveEqual(url, "https://github.com/jupyter-widgets/pythreejs")

    def test_ipywidgets_server(self):
        # https://github.com/pbugnion/ipywidgets_server
        self.assertRaisesNoUrls("ipywidgets_server")

    def test_jupyter_singleton(self):
        url = self.converter.get_vcs("jupyter-singleton")
        self.assertInsensitiveEqual(
            url, "https://github.com/EricMende/jupyter_singleton"
        )

    def test_jupyter_pip(self):
        url = self.converter.get_vcs("jupyter-pip")
        self.assertInsensitiveEqual(url, "https://github.com/jdfreder/jupyter-pip")

    def test_widgetsnbextension(self):
        # https://github.com/jupyter-widgets/ipywidgets/tree/master/widgetsnbextension
        self.assertRaisesNoUrls("widgetsnbextension")

    def test_widgetsnbextension_old(self):
        self.assertRaisesNoUrls("jupyter-js-widgets-nbextension")

    def test_jupyterlab(self):
        url = self.converter.get_vcs("jupyterlab")
        self.assertInsensitiveEqual(url, "https://github.com/jupyterlab/jupyterlab")

    def test_jupyterlab_server(self):
        url = self.converter.get_vcs("jupyterlab-server")
        self.assertInsensitiveEqual(
            url, "https://github.com/jupyterlab/jupyterlab_server"
        )

    def test_jupyterlab_git(self):
        url = self.converter.get_vcs("jupyterlab-git")
        self.assertInsensitiveEqual(url, "https://github.com/jupyterlab/jupyterlab-git")

    def test_jupyterlab_latex(self):
        url = self.converter.get_vcs("jupyterlab-latex")
        self.assertInsensitiveEqual(
            url, "https://github.com/jupyterlab/jupyterlab-latex"
        )

    def test_jupyterlab_nvdashboard(self):
        url = self.converter.get_vcs("jupyterlab-nvdashboard")
        self.assertInsensitiveEqual(
            url, "https://github.com/rapidsai/jupyterlab-nvdashboard"
        )

    def test_jupyterlab_bokeh_server(self):
        url = self.converter.get_vcs("jupyterlab-bokeh-server")
        self.assertInsensitiveEqual(
            url, "https://github.com/ian-r-rose/jupyterlab-bokeh-server"
        )

    def test_databrickslabs_jupyterlab(self):
        url = self.converter.get_vcs("databrickslabs-jupyterlab")
        self.assertInsensitiveEqual(
            url, "https://github.com/databrickslabs/Jupyterlab-Integration"
        )

    def test_jupyterthemes(self):
        url = self.converter.get_vcs("jupyterthemes")
        self.assertInsensitiveEqual(url, "https://github.com/dunovank/jupyter-themes")

    def test_jupyter_contrib_nbextensions(self):
        url = self.converter.get_vcs("jupyter_contrib_nbextensions")
        self.assertInsensitiveEqual(
            url, "https://github.com/ipython-contrib/jupyter_contrib_nbextensions"
        )

    def test_jupyter_nbextensions_configurator(self):
        url = self.converter.get_vcs("jupyter_nbextensions_configurator")
        self.assertInsensitiveEqual(
            url, "https://github.com/jupyter-contrib/jupyter_nbextensions_configurator"
        )

    def test_jupyterlab_pygments(self):
        self.assertRaisesNoUrls("jupyterlab-pygments")

    def test_matplotlib(self):
        url = self.converter.get_vcs("matplotlib")
        self.assertInsensitiveEqual(url, "https://github.com/matplotlib/matplotlib")

    def test_ipympl(self):
        # https://github.com/matplotlib/jupyter-matplotlib
        self.assertRaisesNoUrls("ipympl")

    def test_cycler(self):
        url = self.converter.get_vcs("cycler")
        self.assertInsensitiveEqual(url, "https://github.com/matplotlib/cycler")

    def test_pyviz(self):
        url = self.converter.get_vcs("pyviz")
        self.assertInsensitiveEqual(url, "https://github.com/pyviz/pyviz")

    def test_pyviz_comms(self):
        url = self.converter.get_vcs("pyviz-comms")
        self.assertInsensitiveEqual(url, "https://github.com/holoviz/pyviz_comms")


class TestROS(_TestBase):
    def test_catkin(self):
        url = self.converter.get_vcs("catkin")
        self.assertInsensitiveEqual(url, "https://github.com/ros/catkin")

    def test_catkin_pkg(self):
        url = self.converter.get_vcs("catkin-pkg")
        self.assertInsensitiveEqual(
            url, "https://github.com/ros-infrastructure/catkin_pkg"
        )

    def test_rosdep(self):
        url = self.converter.get_vcs("rosdep")
        self.assertInsensitiveEqual(url, "https://github.com/ros-infrastructure/rosdep")

    def test_rosdistro(self):
        url = self.converter.get_vcs("rosdistro")
        self.assertInsensitiveEqual(url, "https://github.com/ros/rosdistro")

    def test_wstool(self):
        url = self.converter.get_vcs("wstool")
        self.assertInsensitiveEqual(url, "https://github.com/vcstools/wstool")

    def test_rosinstall(self):
        url = self.converter.get_vcs("rosinstall")
        self.assertInsensitiveEqual(url, "https://github.com/vcstools/rosinstall")

    def test_rosinstall_generator(self):
        url = self.converter.get_vcs("rosinstall_generator")
        self.assertInsensitiveEqual(
            url, "https://github.com/ros-infrastructure/rosinstall_generator"
        )


class TestOrange(_TestBase):
    def test_orange3(self):
        url = self.converter.get_vcs("orange3")
        self.assertInsensitiveEqual(url, "https://github.com/biolab/orange3")

    def test_orange(self):
        url = self.converter.get_vcs("orange")
        self.assertInsensitiveEqual(url, "https://bitbucket.org/biolab/orange")

    def test_orange_widget_base(self):
        url = self.converter.get_vcs("orange-widget-base")
        self.assertInsensitiveEqual(url, "https://github.com/biolab/orange-widget-base")

    def test_orange_canvas_core(self):
        url = self.converter.get_vcs("orange_canvas-core")
        self.assertInsensitiveEqual(url, "https://github.com/biolab/orange-canvas-core")

    def test_orange3_ImageAnalytics(self):
        url = self.converter.get_vcs("orange3-ImageAnalytics")
        self.assertInsensitiveEqual(
            url, "https://github.com/biolab/orange3-ImageAnalytics"
        )

    def test_orange3_Recommendation(self):
        url = self.converter.get_vcs("orange3-Recommendation")
        self.assertInsensitiveEqual(
            url, "https://github.com/biolab/orange3-Recommendation"
        )

    def test_orange_text(self):
        url = self.converter.get_vcs("orange-Text")
        self.assertInsensitiveEqual(url, "https://bitbucket.org/biolab/orange-text")

    def test_orange3_text(self):
        url = self.converter.get_vcs("orange3-text")
        self.assertInsensitiveEqual(url, "https://github.com/biolab/orange3-text")

    def test_nonpypi_PyQtTester(self):
        with self.assertRaises(InvalidPackage):
            self.converter.get_vcs("PyQtTester")


class TestEnthought(_TestBase):
    def test_grin(self):
        url = self.converter.get_vcs("grin")
        self.assertInsensitiveEqual(url, "https://github.com/rkern/grin")

    def test_traits(self):
        url = self.converter.get_vcs("traits")
        self.assertInsensitiveEqual(url, "https://github.com/enthought/traits")

    def test_traitsui(self):
        url = self.converter.get_vcs("traitsui")
        self.assertInsensitiveEqual(url, "https://github.com/enthought/traitsui")

    def test_etsdevtools(self):
        url = self.converter.get_vcs("etsdevtools")
        self.assertInsensitiveEqual(url, "https://github.com/enthought/etsdevtools")


class TestGnome(_TestBase):
    def test_pygtk(self):
        url = self.converter.get_vcs("pygtk")
        self.assertInsensitiveEqual(url, "https://gitlab.gnome.org/gnome/pygobject")

    def test_pygobject(self):
        url = self.converter.get_vcs("pygobject")
        self.assertInsensitiveEqual(url, "https://gitlab.gnome.org/gnome/pygobject")

    def test_pyocr(self):
        url = self.converter.get_vcs("pyocr")
        self.assertEqual(url, "https://gitlab.gnome.org/World/OpenPaperwork/pyocr")

    def test_pyinsane2(self):
        url = self.converter.get_vcs("pyinsane2")
        self.assertInsensitiveEqual(url, "https://github.com/openpaperwork/pyinsane")
        # Should be "https://gitlab.gnome.org/World/OpenPaperwork/pyinsane")

    def test_pypillowfight(self):
        url = self.converter.get_vcs("pypillowfight")
        self.assertEqual(url, "https://gitlab.gnome.org/World/OpenPaperwork/libpillowfight")


class TestQt(_TestBase):
    def _test_dip(self):
        url = self.converter.get_vcs("dip")
        self.assertInsensitiveEqual(url, "https://www.riverbankcomputing.com/hg/dip")

    def test_pyqtdeploy(self):
        url = self.converter.get_vcs("pyqtdeploy")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/pyqtdeploy"
        )

    def _test_pyqt_builder(self):
        url = self.converter.get_vcs("PyQt-builder")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/hg/PyQt-builder"
        )

    def test_sip(self):
        url = self.converter.get_vcs("sip")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/sip"
        )

    def test_pyqt4(self):
        url = self.converter.get_vcs("pyqt4")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/pyqt"
        )

    def _test_pyqt4_sip(self):
        url = self.converter.get_vcs("pyqt4-sip")
        self.assertInsensitiveEqual(url, "https://sourceforge.net/projects/pyqt4")

    def test_pyqt5(self):
        # Maybe should be https://sourceforge.net/projects/pyqt5
        url = self.converter.get_vcs("pyqt5")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/pyqt"
        )

    def test_pyqt5_sip(self):
        url = self.converter.get_vcs("pyqt5-sip")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/sip"
        )

    def test_pyqtchart(self):
        url = self.converter.get_vcs("pyqtchart")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/pyqtchart"
        )

    def test_pyqtwebengine(self):
        url = self.converter.get_vcs("pyqtwebengine")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/pyqtwebengine"
        )

    def test_pyqtdatavisualization(self):
        url = self.converter.get_vcs("pyqtdatavisualization")
        self.assertInsensitiveEqual(
            url, "https://www.riverbankcomputing.com/software/pyqtdatavisualization"
        )


class TestFreedesktop(_TestBase):
    def test_pyxdg(self):
        # https://github.com/jayvdb/pypidb/issues/41 should be
        # http://cgit.freedesktop.org/xdg/pyxdg/
        url = self.converter.get_vcs("pyxdg")
        #self.assertInsensitiveEqual(url, "https://github.com/takluyver/pyxdg")
        self.assertInsensitiveEqual(url, "https://gitlab.freedesktop.org/xdg/pyxdg")

    def test_dbus(self):
        url = self.converter.get_vcs("dbus-python")
        self.assertInsensitiveEqual(url, "https://gitlab.freedesktop.org/dbus/dbus-python")

    def test_hid_tools(self):
        url = self.converter.get_vcs("hid-tools")
        self.assertInsensitiveEqual(url, "https://gitlab.freedesktop.org/libevdev/hid-tools")

    def test_libevdev(self):
        url = self.converter.get_vcs("libevdev")
        self.assertInsensitiveEqual(url, "https://gitlab.freedesktop.org/libevdev/python-libevdev")

    def test_xpyb(self):
        url = self.converter.get_vcs("xpyb")
        self.assertInsensitiveEqual(url, "https://gitlab.freedesktop.org/xcb/xpyb")


class TestMaxmind(_TestBase):
    def test_GeoIP(self):
        url = self.converter.get_vcs("GeoIP")
        self.assertInsensitiveEqual(url, "https://github.com/maxmind/GeoIP-api-python")

    def test_GeoIP2(self):
        url = self.converter.get_vcs("GeoIP2")
        self.assertInsensitiveEqual(url, "https://github.com/maxmind/GeoIP2-python")

    def test_minfraud(self):
        url = self.converter.get_vcs("minfraud")
        self.assertInsensitiveEqual(url, "https://github.com/maxmind/minfraud-api-python")



import base64
import json
import os

from appdirs import user_cache_dir
from logging_helper import setup_logging

from ._cache import get_file_cache_session
from ._similarity import normalize

gh_session = get_file_cache_session("gh")

logger = setup_logging()


class GitHubAPIMessage(Exception):
    pass


def api_call(endpoint, method, field_name=None):
    endpoint = endpoint.lstrip("/")
    headers = {}

    api_token = os.getenv("GITHUB_API_TOKEN")
    if api_token:  # pragma: no cover
        logger.info("Using API token")
        headers["Authorization"] = "token {}".format(api_token)

    gh_session.headers.update(headers)

    if method == "GET":
        r = gh_session.get(
            "https://api.github.com/{}".format(endpoint), headers=headers
        )

        logger.info("r status code {}: {}".format(r.status_code, r.reason))

        if r.status_code in [404]:
            raise GitHubAPIMessage("Not Found")

        r.raise_for_status()

        try:
            rj = r.json()
        except json.JSONDecodeError:  # pragma: no cover
            logger.error("failed to decode {}:\n{}".format(endpoint, r.text))
            raise

        if list(rj.keys()) == ["message", "documentation_url"]:
            raise GitHubAPIMessage(rj["message"])

        if field_name:
            if field_name in rj:
                return rj[field_name]
            else:
                logger.error("{} not in {}".format(field_name, rj.keys()))
        else:
            try:
                return r.json()
            except json.JSONDecodeError as e:  # pragma: no cover
                logger.error("failed to decode {}: {}\n{}".format(endpoint, e, r.text))
                raise


def check_repo(url_or_slug):
    if url_or_slug.startswith("https://github.com/"):
        slug = url_or_slug[len("https://github.com/") :]
    else:
        slug = url_or_slug

    try:
        r = api_call(endpoint="/repos/{}".format(slug), method="GET")
    except GitHubAPIMessage as e:
        if str(e) == "Not Found":
            return
        raise
    return r


def get_repo_top_language(slug):  # pragma: no cover
    try:
        languages = api_call(endpoint="/repos/{}/languages".format(slug), method="GET")
    except GitHubAPIMessage:
        raise
    except (NameError, SystemExit) as e:  # pragma: no cover
        logger.warning("get_repo_setuppy gh {} failed {}".format(slug, e))
        return
    try:
        return next(iter(languages))
    except StopIteration:
        return False
    logger.info("top lang = {}".format(top_language))


def repo_is_python(slug):  # pragma: no cover
    top_language = get_repo_top_language(slug)
    logger.info("{}: top lang = {}".format(slug, top_language))
    return top_language == "Python"


def api_get_file(slug, filename):  # pragma: no cover
    try:
        c = api_call(
            endpoint="/repos/{}/contents/{}".format(slug, filename),
            method="GET",
            field_name="content",
        )
    except GitHubAPIMessage as e:
        if str(e) == "Not Found":
            return
        else:
            raise
    except Exception as e:  # pragma: no cover
        logger.warning("gh {} failed {}".format(slug, e))
        raise
    if not c:
        return
    return base64.b64decode(c).decode("utf-8")


def raw_get_file(slug, filename):
    url = "https://raw.githubusercontent.com/{}/master/{}".format(slug, filename)
    r = gh_session.get(url)
    if r.status_code == 404:
        return
    r.raise_for_status()
    try:
        return r.content.decode(r.apparent_encoding)
    except Exception:
        return r.content.decode("utf-8")


def get_repo_setuppy(url_or_slug, matches, filenames=None):
    if url_or_slug.startswith("https://github.com/"):  # pragma: no cover
        slug = url_or_slug[len("https://github.com/") :]
    else:
        slug = url_or_slug

    if not isinstance(matches, list):
        matches = [matches]

    setuppy = None
    if not filenames:
        filenames = ["setup.py", "pyproject.toml", "setup.cfg"]

    for filename in filenames:
        try:
            c = raw_get_file(slug, filename)
        except Exception as e:
            logger.warning("gh raw {} {}: failed {}".format(slug, filename, e))
            continue
        if not c:
            continue
        c = normalize(c)
        if not setuppy:
            setuppy = c
        if all(match in c for match in matches):
            return c
        try:
            logger.info("{} not found in {}".format(matches, c))
        except Exception:  # pragma: no cover
            logger.info("{} not found in {}".format(matches, filename))

    logger.warning("{}: {} not found in {}".format(slug, matches, filenames))
    return setuppy



import unittest

from pypidb._rtd import AuthenticationError, ReadtheDocs, get_repo
from tests.utils import _TestBase


class TestGitHubRepo(_TestBase):
    def test_moneta(self):
        rv = get_repo("moneta")
        self.assertEqual(rv, "https://github.com/d9pouces/Moneta")

        url = get_repo("https://moneta.readthedocs.io/")
        self.assertEqual(url, "https://github.com/d9pouces/Moneta")

        url = get_repo("https://moneta.readthedocs.org/")
        self.assertEqual(url, "https://github.com/d9pouces/Moneta")

        url = get_repo("http://moneta.readthedocs.io/")
        self.assertEqual(url, "https://github.com/d9pouces/Moneta")

        url = get_repo("http://moneta.readthedocs.org/")
        self.assertEqual(url, "https://github.com/d9pouces/Moneta")

    def test_http_upgrade_cvxopt(self):
        url = get_repo("cvxopt")
        self.assertEqual(url, "https://github.com/cvxopt/cvxopt")

    def test_uwsgi_docs(self):
        url = get_repo("uwsgi-docs")
        self.assertInsensitiveEqual(url, "https://github.com/unbit/uwsgi")

    def test_mailgun(self):
        url = get_repo("mailgun")
        self.assertInsensitiveEqual(url, "https://github.com/Bogardo/Mailgun")

    def test_simplekml(self):
        url = get_repo("simplekml")
        self.assertInsensitiveEqual(url, "https://github.com/eisoldt/simplekml")

    def test_mailgun_docs(self):
        try:
            url = ReadtheDocs().get_project("mg-documentation")
            self.assertInsensitiveEqual(
                url, "https://github.com/mailgun/documentation.git"
            )
        except AuthenticationError:
            pass

        url = get_repo("mg-documentation")
        self.assertInsensitiveEqual(url, "https://github.com/mailgun/documentation")

    def test_jsonschema_objects(self):
        url = get_repo("python-jsonschema-objects")
        self.assertEqual(url, "https://github.com/cwacek/python-jsonschema-objects")

        url = self.converter.get_vcs("python-jsonschema-objects")
        self.assertEqual(url, "https://github.com/cwacek/python-jsonschema-objects")

    def test_mwlib(self):
        url = get_repo("mwlib")
        self.assertEqual(url, "https://github.com/pediapress/mwlib")

    def test_tidy(self):
        url = get_repo("tidy")
        self.assertEqual(url, "https://github.com/xywei/tidy")

    def test_ccdproc(self):
        url = get_repo("ccdproc")
        self.assertEqual(url, "https://github.com/astropy/ccdproc")

    def test_pygobject(self):
        url = get_repo("pygobject")
        self.assertIsNone(url)


class TestRedirect(_TestBase):
    def test_sqlalchemy(self):
        try:
            url = ReadtheDocs().get_project("sqlalchemy")
        except AuthenticationError:
            raise unittest.SkipTest("auth failure prevents test logic")

        self.assertInsensitiveEqual(
            url, "https://github.com/zzzeek/redirectthedocs.git"
        )

    def test_sqlalchemy_ignored(self):
        url = get_repo("sqlalchemy")
        self.assertIsNone(url)


class TestReadthedocsHosted(_TestBase):
    def test_datarobot(self):
        url = get_repo(
            "https://datarobot-public-api-client.readthedocs-hosted.com/",
            version="v2.19.0",
        )
        self.assertInsensitiveEqual(
            url, "https://github.com/datarobot/public_api_client"
        )

        url = get_repo("https://datarobot-public-api-client.readthedocs-hosted.com/")
        self.assertIsNone(url)

        url = get_repo(
            "https://datarobot-public-api-client.readthedocs-hosted.com/en/v2.19.0/"
        )
        self.assertInsensitiveEqual(
            url, "https://github.com/datarobot/public_api_client"
        )

        url = self.converter.get_vcs("datarobot")
        self.assertInsensitiveEqual(
            url, "https://github.com/datarobot/public_api_client"
        )

    def test_plaidml(self):
        url = get_repo("plaidml")
        self.assertInsensitiveEqual(url, "https://github.com/plaidml/plaidml")

    def test_plaidml_vertexai(self):
        url = get_repo("https://vertexai-plaidml.readthedocs-hosted.com/")
        self.assertInsensitiveEqual(url, "https://github.com/plaidml/plaidml")

    def test_mapbox(self):
        url = get_repo("https://mapbox-mapbox.readthedocs-hosted.com/")
        self.assertInsensitiveEqual(url, "https://github.com/mapbox/mapbox-sdk-py")

        url = self.converter.get_vcs("mapbox")
        self.assertInsensitiveEqual(url, "https://github.com/mapbox/mapbox-sdk-py")

    def test_mapboxgl(self):
        url = get_repo("https://mapbox-mapboxgl-jupyter.readthedocs-hosted.com/")
        self.assertInsensitiveEqual(url, "https://github.com/mapbox/mapboxgl-jupyter")

        url = self.converter.get_vcs("mapboxgl")
        self.assertInsensitiveEqual(url, "https://github.com/mapbox/mapboxgl-jupyter")

    def test_scanpy(self):
        url = get_repo("https://icb-scanpy.readthedocs-hosted.com/")
        self.assertEqual(url, "https://github.com/theislab/scanpy")

        url = self.converter.get_vcs("scanpy")
        self.assertEqual(url, "https://github.com/theislab/scanpy")

    def test_message_ix(self):
        url = get_repo(
            "https://iiasa-energy-program-message-ix.readthedocs-hosted.com/"
        )
        self.assertEqual(url, "https://github.com/iiasa/message_ix")

        url = self.converter.get_vcs("message-ix")
        self.assertEqual(url, "https://github.com/iiasa/message_ix")

    def test_cognite_docs(self):
        url = get_repo(
            "https://cognite-docs.readthedocs-hosted.com/", strip_docs_suffix=False
        )
        self.assertEqual(url, "https://github.com/cognitedata/cognite-python-docs")

        url = self.converter.get_vcs("cognite-sdk")
        self.assertEqual(url, "https://github.com/cognitedata/cognite-sdk-python")

    def test_methylcheck(self):
        url = get_repo("https://life-epigenetics-methylcheck.readthedocs-hosted.com/")
        self.assertEqual(url, "https://github.com/LifeEGX/methylcheck")

        url = self.converter.get_vcs("methylcheck")
        self.assertInsensitiveEqual(url, "https://github.com/LifeEGX/methylcheck")

    def test_arundo_tsaug(self):
        url = get_repo("https://arundo-tsaug.readthedocs-hosted.com")
        self.assertEqual(url, "https://github.com/arundo/tsaug")

        url = self.converter.get_vcs("tsaug")
        self.assertEqual(url, "https://github.com/arundo/tsaug")

    def test_arundo_adtk(self):
        url = get_repo("https://arundo-adtk.readthedocs-hosted.com/")
        self.assertEqual(url, "https://github.com/arundo/adtk")

        url = self.converter.get_vcs("adtk")
        self.assertEqual(url, "https://github.com/arundo/adtk")

    def test_pytext(self):
        url = get_repo("https://pytext-pytext.readthedocs-hosted.com/")
        self.assertEqual(url, "https://github.com/facebookresearch/pytext")

    def test_lime_newsletter(self):
        url = get_repo("https://lime-lime-newsletter.readthedocs-hosted.com/")
        self.assertEqual(url, "https://github.com/Lundalogik/lime-newsletter")

    def test_leapyear_python(self):
        url = get_repo(
            "https://leapyear-python-docs.readthedocs-hosted.com/", version="2.9.1"
        )
        self.assertIsNone(url)  # Auth needed

    def _test_other(self):
        url = get_repo("https://totvslabs-pycarol.readthedocs-hosted.com/")
        self.assertIsNone(url)
        url = get_repo("https://scalr-athena.readthedocs-hosted.com")  # github
        self.assertIsNotNone(url)


class TestBitBucketRepo(_TestBase):
    def test_feedcache(self):
        url = get_repo("feedcache")
        self.assertEqual(url, "https://bitbucket.org/dhellmann/feedcache")

    def test_hudya_docs(self):
        url = get_repo(
            "https://hudya-docs.readthedocs-hosted.com/", strip_docs_suffix=False
        )
        self.assertEqual(url, "https://bitbucket.org/hudya/docs")

    def test_django_portlets(self):
        url = get_repo("django-portlets")
        self.assertEqual(url, "https://bitbucket.org/diefenbach/django-portlets")

    def test_django_portlets_redirect(self):
        url = self.converter.get_vcs("django-portlets")
        self.assertEqual(url, "https://github.com/diefenbach/django-portlets")



from netrc import netrc

from logging_helper import setup_logging

logger = setup_logging()


def _get_token(hostname):
    try:
        conf = netrc()
        token = conf.hosts.get(hostname, [None])[0]
        return token
    except Exception as e:  # pragma: no cover
        logger.error("get_token e: {}".format(e))



from ._cache import get_file_cache
from ._compat import urlsplit

web_session = get_file_cache("web")


def _pagure_io(url):
    p = urlsplit(url)
    path = p.path[1:]
    path1, _, other = path.partition("/")
    path2, _, other = other.partition("/")

    if path2:
        r = web_session.get("https://pagure.io/api/0/{}/{}".format(path1, path2))
        if r.status_code == 200:
            data = r.json()
            if isinstance(data, dict) and "fullname" in data:
                return "https://pagure.io/{}/{}".format(path1, path2)

    r = web_session.get("https://pagure.io/api/0/{}".format(path1))
    if r.status_code == 200:
        data = r.json()
        if isinstance(data, dict) and "fullname" in data:
            return "https://pagure.io/{}".format(path1)

    return
